﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace FreqAnalizer
{
    public partial class FreqAnaLog : Form
    {
        int[] baud_rate_select_datas = new int[12] { 1200, 2400, 3600, 9600, 19200, 38400, 57600, 76800, 115200, 230400, 460800, 921600 };
        const int baud_rate_default_select_idx = 8;
        const int SERIAL_RECEIVED_BUFF_SIZE = 0x1000;
        byte[] serial_received_buff = new byte[SERIAL_RECEIVED_BUFF_SIZE];
        int serial_rcv_data_read_idx = 0;
        int serial_rcv_data_write_idx = 0;
        bool serial_rcv_data_full = false;
        //const int FREQ_SET_VAL_MIN = 10;
        const int FREQ_SET_VAL_MIN = 10;
        const int FREQ_SET_VAL_MAX = 10000000;

        bool freq_manual_loop_set_flag = false;
        int freq_manual_loop_send_interval_counter = 0;
        const int FREQ_MANUAL_LOOP_SEND_INTERVAL = 1000;

        bool freq_auto_set_flag = false;
        int freq_auto_set_freq_val = 0;
        int freq_auto_set_freq_val_next = 0;

        //
        int freq_sweep_step_val= 10;
        double freq_auto_set_freq_val_next1 = 0;
        double db_tmp_level_val = 10;
        double db_tmp_level = 10;
        double sweep_start_freq = FREQ_SET_VAL_MIN;
        double sweep_stop_freq = FREQ_SET_VAL_MAX;
        // add

        int freq_manual_set_freq_val = 0;
        bool freq_auto_set_data_rcv_flag = false;

        // Log file
        bool logfile_out_flag = false;
        StreamWriter sw_serial_log = null;
        const string LOG_FILE_NAME = "\\FREQ_LOG_";
        const string LOG_FILE_EXTEN = ".csv";
        string[] LOG_FILE_HEADER = new string[2] {"freq","level"};

        public FreqAnaLog()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {
#if DEBUG
#else
                this.Size = new System.Drawing.Size(925, 575);

                groupBox1.Visible = false;
#endif

                // 透過などのために親設定
                gbx_main.Parent = this;

                for (int fi = 0; fi < 32; fi++)
                {
                    cmbbx_port.Items.Add(string.Format("COM{0}", fi + 1));
                }
                cmbbx_port.SelectedIndex = 0;
                for (int fi = 0; fi < baud_rate_select_datas.Length; fi++)
                {
                    cmbbx_baudrate.Items.Add(baud_rate_select_datas[fi].ToString());
                }
                cmbbx_baudrate.SelectedIndex = baud_rate_default_select_idx;

                // チャート設定
                // 1.Seriesの追加
                chart1.Series.Clear();
                chart1.Series.Add("level");
                chart1.Series[0].Color = Color.Orange;
                chart1.Series[0].IsVisibleInLegend = false;
                chart1.Series[0].IsValueShownAsLabel = false;
                // 2.グラフのタイプの設定
                chart1.Series["level"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.Line;
                //軸ラベルの設定
                chart1.ChartAreas[0].AxisX.Title = "周波数[Hz]";
                chart1.ChartAreas[0].AxisY.Title = "レベル[dB]";

                //X軸最小値、最大値、目盛間隔の設定
                chart1.ChartAreas[0].AxisX.Minimum = FREQ_SET_VAL_MIN;
                chart1.ChartAreas[0].AxisX.Maximum = FREQ_SET_VAL_MAX;
                //chart1.ChartAreas[0].AxisX.Interval = 1000000;
                //Y軸最小値、最大値、目盛間隔の設定
                chart1.ChartAreas[0].AxisY.Minimum = -60;
                chart1.ChartAreas[0].AxisY.Maximum = +10;
                chart1.ChartAreas[0].AxisY.Interval = 10;
                // ダミーデータ
                chart1.Series[0].Points.AddXY(10, 0);

                FormUpdateTimer.Enabled = true;
            }
            catch
            {
            }
        }

        private void FreqAnaLog_FormClosed(object sender, FormClosedEventArgs e)
        {
            try
            {
                if (serialPort1.IsOpen == true)
                {
                    serialPort1.Close();
                }
            }
            catch
            {
            }
        }

        private void btn_serial_open_Click(object sender, EventArgs e)
        {
            bool error_flag = false;
            byte[] send_data = new byte[64];
            try
            {
                if (serialPort1.IsOpen == false)
                {
                    serialPort1.PortName = cmbbx_port.SelectedItem.ToString();
                    if (0 <= cmbbx_baudrate.SelectedIndex && cmbbx_baudrate.SelectedIndex < baud_rate_select_datas.Length)
                    {
                        serialPort1.BaudRate = baud_rate_select_datas[cmbbx_baudrate.SelectedIndex];
                    }
                    else
                    {
                        error_flag = true;
                    }


                    if (error_flag == false)
                    {
                        try
                        {
                            serialPort1.WriteTimeout = 500;
                            serialPort1.ReadTimeout = 500;

                            serialPort1.Open();
                            if(serialPort1.IsOpen == true)
                            {
                                //backgroundWorker1.RunWorkerAsync();

                                send_data[0] = 0x53;//S
                                send_data[1] = 0x54;//T
                                send_data[2] = 0x45;//E

                                try
                                {
                                    serialPort1.Write(send_data, 0, send_data.Length);

                                    btn_serial_open.Enabled = false;
                                    btn_serial_close.Enabled = true;
                                    btn_serial_send1.Enabled = true;
                                    btn_serial_send2.Enabled = true;

                                    btn_sweep_start.Enabled = true;
                                    btn_sweep_stop.Enabled = false;
                                    btn_logfile_start.Enabled = true;
                                    btn_logfile_stop.Enabled = false;
                                }
                                catch
                                {
                                    serialPort1.Close();
                                    MessageBox.Show("シリアル通信エラー", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                                }
                            }
                            else
                            {
                                MessageBox.Show("シリアルポートオープンエラー", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            }
                        }
                        catch
                        {
                            MessageBox.Show("シリアルポートオープンエラー", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    else
                    {
                        MessageBox.Show("シリアルポート　設定値エラー", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
            catch
            {
            }
        }

        private void btn_serial_close_Click(object sender, EventArgs e)
        {
            try
            {
                //backgroundWorker1.CancelAsync();

                if (serialPort1.IsOpen == true)
                {
                    serialPort1.Close();

                    btn_serial_open.Enabled = true;
                    btn_serial_close.Enabled = false;
                    btn_serial_send1.Enabled = false;
                    btn_serial_send2.Enabled = false;

                    sweep_stop();
                    btn_sweep_start.Enabled = false;
                    btn_sweep_stop.Enabled = false;
                    my_logfile_output_stop();
                    btn_logfile_start.Enabled = false;
                    btn_logfile_stop.Enabled = false;
                }
            }
            catch
            {
            }
        }

        private void btn_serial_send1_Click(object sender, EventArgs e)
        {
            byte[] send_data = new byte[64];
            try
            {
                if (serialPort1.IsOpen == true)
                {
                    send_data[0] = 0x53;//S
                    send_data[1] = 0x54;//T
                    send_data[2] = 0x45;//E

                    serialPort1.Write(send_data, 0, send_data.Length);
                }
            }
            catch
            {
            }
        }

        private void btn_serial_send2_Click(object sender, EventArgs e)
        {
            try
            {
                int set_freq = (int)num_set_freq.Value;
                freq_manual_set_freq_val = set_freq;

                if(freq_manual_loop_set_flag == true)
                {
                    freq_manual_loop_set_flag = false;
                    btn_serial_send2.Text = "送信";

                    num_set_freq.Enabled = true;
                    chkbx_freq_manual_loop.Enabled = true;

                    btn_sweep_start.Enabled = true;
                    btn_sweep_stop.Enabled = false;
                }
                else
                {
                    if (chkbx_freq_manual_loop.Checked == true)
                    {
                        // 繰り返し
                        freq_manual_loop_set_flag = true;
                        freq_manual_loop_send_interval_counter = 0;
                        btn_serial_send2.Text = "送信停止";

                        num_set_freq.Enabled = false;
                        chkbx_freq_manual_loop.Enabled = false;

                        btn_sweep_start.Enabled = false;
                        btn_sweep_stop.Enabled = false;
                    }
                    else
                    {
                        my_serial_freq_set_command_send(set_freq);
                    }
                }
            }
            catch
            {
            }
        }

        private void my_serial_freq_set_command_send(int p_set_freq)
        {
            try
            {
                byte[] send_data = new byte[64];
                try
                {
                    if (FREQ_SET_VAL_MIN <= p_set_freq && p_set_freq <= FREQ_SET_VAL_MAX)
                    {
                        if (serialPort1.IsOpen == true)
                        {
                            int set_freq = p_set_freq;

                            send_data[0] = 0x53;//S
                            send_data[1] = 0x4E;//N
                            send_data[2] = (byte)(set_freq & 0xFF);         //F1
                            send_data[3] = (byte)((set_freq >> 8) & 0xFF);  //F2
                            send_data[4] = (byte)((set_freq >> 16) & 0xFF); //F3
                            send_data[5] = (byte)((set_freq >> 24) & 0xFF); //F4
                            send_data[6] = 0x45;//E

                            serialPort1.Write(send_data, 0, send_data.Length);
                        }
                    }
                }
                catch
                {
                }
            }
            catch
            {
            }
        }

        private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                while (backgroundWorker1.CancellationPending == false)
                {
                    while (serial_rcv_data_read_idx != serial_rcv_data_write_idx || serial_rcv_data_full == true)
                    {
                    }
                }
            }
            catch
            {
            }
        }

        private void FormUpdateTimer_Tick(object sender, EventArgs e)
        {
            try
            {

                byte[] tmp_serial_data = new byte[5];
                int tmp_read_idx = serial_rcv_data_read_idx;
                bool tmp_full = serial_rcv_data_full;
                int rcv_data_count = 0;
                string serial_debug_str = "";
                while (serial_rcv_data_read_idx != serial_rcv_data_write_idx || serial_rcv_data_full == true)
                {
                    if (serial_received_buff[tmp_read_idx] == 0x4D)
                    {   // 先頭M
                        rcv_data_count = 0;
                        tmp_serial_data[rcv_data_count++] = serial_received_buff[tmp_read_idx++];
                        tmp_read_idx &= (SERIAL_RECEIVED_BUFF_SIZE - 1);
                        tmp_full = false;

                        while (tmp_read_idx != serial_rcv_data_write_idx || tmp_full == true)
                        {
                            tmp_serial_data[rcv_data_count++] = serial_received_buff[tmp_read_idx++];
                            tmp_read_idx &= (SERIAL_RECEIVED_BUFF_SIZE - 1);
                            tmp_full = false;

                            if (rcv_data_count >= tmp_serial_data.Length)
                            {
                                break;
                            }
                        }

                        if (rcv_data_count == tmp_serial_data.Length)
                        {   // データ数OK
                            //データチェック M,N,L1,L2,E
                            if (tmp_serial_data[0] == 0x4D && tmp_serial_data[1] == 0x4E && tmp_serial_data[4] == 0x45)
                            {
                                int tmp_level = 0;
                                tmp_level = tmp_serial_data[3];
                                tmp_level = (tmp_level << 8) | (tmp_serial_data[2] & 0x7F);
                                if ((tmp_serial_data[2] & 0x80) != 0)
                                {
                                    tmp_level += 128;
                                }
                                // デシベルに変換
                                //double db_tmp_level = tmp_level;
                                db_tmp_level = tmp_level;
                                db_tmp_level = (0.04 * db_tmp_level) - 81.2;
                                string tmp_level_str = string.Format("{0}", db_tmp_level);

                                string out_str = "";

                                db_tmp_level_val = (int)db_tmp_level;
                                textbx_input_db.Text = db_tmp_level_val.ToString();



                                if (freq_auto_set_data_rcv_flag == true)
                                {   // スリーブ中
                                    out_str = "\"" + freq_auto_set_freq_val.ToString() + "\",\"" + tmp_level_str + "\"";


                                    //グラフのデータを追加
                                    chart1.Series[0].Points.AddXY((double)freq_auto_set_freq_val, db_tmp_level);
                                    //chart1.Series[0].Points.AddXY(freq_auto_set_freq_val.ToString(), textbx_input_db.Text);
                                }
                                else
                                {   // 周波数手動設定
                                    out_str = "\"" + freq_manual_set_freq_val.ToString() + "\",\"" + tmp_level_str + "\"";
                                }

                                if (logfile_out_flag == true && sw_serial_log != null)
                                {   // ログファイルへ出力
                                    sw_serial_log.WriteLine(out_str);
                                }

                                serial_debug_str += "\n";
                                if (chkbx_serial_RX_Ascii.Checked == true)
                                {
                                    serial_debug_str += Encoding.ASCII.GetString(tmp_serial_data, 0, rcv_data_count);
                                }
                                else
                                {
                                    for (int fi = 0; fi < rcv_data_count; fi++)
                                    {
                                        serial_debug_str += string.Format(" {0:X2}", tmp_serial_data[fi]);
                                    }
                                }

                                freq_auto_set_data_rcv_flag = false;

                                // 次のデータへ
                                serial_rcv_data_read_idx = tmp_read_idx;
                                serial_rcv_data_full = false;
                            }
                            else
                            {   // データ不正 次のデータへ
                                if (chkbx_serial_RX_Ascii.Checked == true)
                                {
                                    serial_debug_str += Encoding.ASCII.GetString(serial_received_buff, serial_rcv_data_read_idx, 1);
                                }
                                else
                                {
                                    serial_debug_str += string.Format(" {0:X2}", serial_received_buff[serial_rcv_data_read_idx]);
                                }

                                serial_rcv_data_read_idx++;
                                serial_rcv_data_read_idx &= (SERIAL_RECEIVED_BUFF_SIZE - 1);
                                serial_rcv_data_full = false;
                            }
                        }
                        else
                        {   // データ数が足りない
                            break;
                        }
                    }
                    else
                    {   // 先頭Mでない 次のデータへ
                        if (chkbx_serial_RX_Ascii.Checked == true)
                        {
                            serial_debug_str += Encoding.ASCII.GetString(serial_received_buff, serial_rcv_data_read_idx, 1);
                        }
                        else
                        {
                            serial_debug_str += string.Format(" {0:X2}", serial_received_buff[serial_rcv_data_read_idx]);
                        }

                        serial_rcv_data_read_idx++;
                        serial_rcv_data_read_idx &= (SERIAL_RECEIVED_BUFF_SIZE - 1);
                        serial_rcv_data_full = false;
                    }
                }
                if (serial_debug_str != "")
                {
                    rtxtbx_serial_RX.Focus();
                    rtxtbx_serial_RX.AppendText(serial_debug_str);
                }






                if (freq_auto_set_flag == true && freq_auto_set_data_rcv_flag == false)
                {
                    if (sweep_start_freq <= freq_auto_set_freq_val_next && freq_auto_set_freq_val_next <= sweep_stop_freq)
                    {
                        freq_auto_set_freq_val = (int)freq_auto_set_freq_val_next;
                        txtbx_auto_set_freq.Text = freq_auto_set_freq_val.ToString();
                        db_tmp_level_val = (int)db_tmp_level;
                        textbx_input_db.Text = db_tmp_level_val.ToString();
                        my_serial_freq_set_command_send(freq_auto_set_freq_val);
                        freq_auto_set_data_rcv_flag = true;

                        int set_freq = (int)num_end_freq.Value;
                        if (freq_auto_set_freq_val >= set_freq)
                        {   // 次の周期で終了
                                freq_auto_set_flag = false;
                        }
                        else
                        {
                            double tmp_freq = freq_auto_set_freq_val_next1;  //*******loop NG
                            tmp_freq = tmp_freq + (Math.Pow(10, Math.Log10(tmp_freq))) / (int)freq_sweep_step_val;
                            if (sweep_start_freq <= (int)tmp_freq && tmp_freq <= sweep_stop_freq)
                            {
                                freq_auto_set_freq_val_next1 = tmp_freq;
                                freq_auto_set_freq_val_next = (int)tmp_freq;
                            }
                            else
                            {
                                freq_auto_set_freq_val_next = (int)sweep_stop_freq;
                            }
                        }
                    }
                    else
                    {
                        freq_auto_set_flag = false;
                    }

                    if(freq_auto_set_flag == false)
                    {
                        btn_sweep_start.Enabled = true;
                        btn_sweep_stop.Enabled = false;

                        btn_serial_send2.Enabled = true;
                    }
                }

                if(freq_manual_loop_set_flag == true)
                {
                    if (freq_manual_loop_send_interval_counter == 0)
                    {
                        freq_manual_loop_send_interval_counter = FREQ_MANUAL_LOOP_SEND_INTERVAL / FormUpdateTimer.Interval;

                        my_serial_freq_set_command_send(freq_manual_set_freq_val);
                    }
                    else
                    {
                        freq_manual_loop_send_interval_counter--;
                    }
                }

            }
            catch
            {
            }
        }

        private void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            try
            {
                if(serialPort1.BytesToRead > 0)
                {
                    byte[] tmp_read_buff = new byte[serialPort1.BytesToRead];
                    serialPort1.Read(tmp_read_buff, 0, serialPort1.BytesToRead);

                    for (int fi = 0; fi < tmp_read_buff.Length; fi++)
                    {
                        if (serial_rcv_data_full == false)
                        {
                            serial_received_buff[serial_rcv_data_write_idx++] = tmp_read_buff[fi];

                            serial_rcv_data_write_idx &= (SERIAL_RECEIVED_BUFF_SIZE - 1);
                            if (serial_rcv_data_read_idx == serial_rcv_data_write_idx)
                            {
                                serial_rcv_data_full = true;
                            }
                        }
                    }
                }
            }
            catch
            {
            }
        }

        private void btn_serial_rcv_log_clear_Click(object sender, EventArgs e)
        {
            try
            {
                rtxtbx_serial_RX.Text = "";
            }
            catch
            {
            }
        }

        private void sweep_start()
        {
            try
            {
                int sweep_freq = (int)num_sweep_step.Value;
                freq_sweep_step_val = sweep_freq;

                if(num_start_freq.Value > num_end_freq.Value)
                {
                    num_end_freq.Value = num_start_freq.Value;
                }
                sweep_start_freq = (double)num_start_freq.Value;
                sweep_stop_freq = (double)num_end_freq.Value;

                //X軸最小値、最大値、目盛間隔の設定
                chart1.ChartAreas[0].AxisX.Minimum = sweep_start_freq;
                chart1.ChartAreas[0].AxisX.Maximum = sweep_stop_freq;

                int set_freq = (int)sweep_start_freq;

                freq_auto_set_freq_val = set_freq;
                freq_auto_set_freq_val_next = set_freq;
                freq_auto_set_freq_val_next1 = set_freq;  //add

                freq_auto_set_data_rcv_flag = false;
                freq_auto_set_flag = true;
            }
            catch
            {

            }
        }
        private void sweep_stop()
        {
            try
            {
                freq_auto_set_flag = false;
            }
            catch
            {

            }
        }
        private void btn_sweep_start_Click(object sender, EventArgs e)
        {

            try
            {
                sweep_start();

                btn_sweep_start.Enabled = false;
                btn_sweep_stop.Enabled = true;
                btn_serial_send2.Enabled = false;


                chart1.ChartAreas[0].AxisX.IsLogarithmic = false;
                // 1.Seriesの追加
                chart1.Series.Clear();
                chart1.Series.Add("level");
                // 2.グラフのタイプの設定
                chart1.Series["level"].ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.Line;
                //軸ラベルの設定
                chart1.Series[0].Color = Color.Orange;
                chart1.ChartAreas[0].AxisX.Title = "周波数[Hz]";
                chart1.ChartAreas[0].AxisY.Title = "レベル[dB]";
                //chart1.ChartAreas[0].AxisX.IsLogarithmic = true;  //error 20161208


                //X軸最小値、最大値、目盛間隔の設定
                //chart1.ChartAreas[0].AxisX.Minimum = 10;
                //chart1.ChartAreas[0].AxisX.Maximum = 100000;
                //chart1.ChartAreas[0].AxisX.Interval = 1000000;
                //Y軸最小値、最大値、目盛間隔の設定
                chart1.ChartAreas[0].AxisY.Minimum = -60;
                chart1.ChartAreas[0].AxisY.Maximum = +10;
                chart1.ChartAreas[0].AxisY.Interval = 10;

                // ダミーデータ
                chart1.Series[0].Points.AddXY(10, 0);
                chart1.ChartAreas[0].AxisX.IsLogarithmic = true;

            }
            catch
            {
            }
        }

        private void btn_sweep_stop_Click(object sender, EventArgs e)
        {
            try
            {
                btn_sweep_start.Enabled = true;
                btn_sweep_stop.Enabled = false;
                btn_serial_send2.Enabled = true;

                sweep_stop();
            }
            catch
            {
            }
        }

        private void my_logfile_output_start()
        {
            int fi;
            string outStr = "";
            try
            {
                string log_file_path = "";
                string file_datetime = "";

                file_datetime = string.Format("{0:0000}{1:00}{2:00}_{3:00}{4:00}{5:00}", DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);
                log_file_path = System.Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + LOG_FILE_NAME + file_datetime + LOG_FILE_EXTEN;

                try
                {
                    sw_serial_log = new StreamWriter(log_file_path);

                    // ヘッダ
                    for (fi = 0; fi < LOG_FILE_HEADER.Length; fi++)
                    {
                        if (fi >= 1)
                        {
                            outStr += ",";
                        }
                        outStr += "\"" + LOG_FILE_HEADER[fi] + "\"";
                    }
                    sw_serial_log.WriteLine(outStr);

                    btn_logfile_start.Enabled = false;
                    btn_logfile_stop.Enabled = true;

                    logfile_out_flag = true;
                }
                catch
                {
                    if (sw_serial_log != null)
                    {
                        sw_serial_log.Close();
                        sw_serial_log.Dispose();
                        sw_serial_log = null;
                    }
                    btn_logfile_start.Enabled = true;
                    btn_logfile_stop.Enabled = false;
                }

            }
            catch
            {
            }
        }
        private void my_logfile_output_stop()
        {
            try
            {
                if (sw_serial_log != null)
                {
                    sw_serial_log.Close();
                    sw_serial_log.Dispose();
                    sw_serial_log = null;
                }

                logfile_out_flag = false;
            }
            catch
            {
            }
        }

        private void btn_logfile_start_Click(object sender, EventArgs e)
        {
            try
            {
                my_logfile_output_start();
            }
            catch
            {
            }
        }

        private void btn_logfile_stop_Click(object sender, EventArgs e)
        {
            try
            {
                btn_logfile_start.Enabled = true;
                btn_logfile_stop.Enabled = false;

                my_logfile_output_stop();
            }
            catch
            {
            }
        }

    }
}
